c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     #################################################################
c     ##                                                             ##
c     ##  subroutine bounds  --  check periodic boundary conditions  ##
c     ##                                                             ##
c     #################################################################
c
c
c     "bounds" finds the center of mass of each molecule and
c     translates any stray molecules back into the periodic box
c
c
      subroutine bounds
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'boxes.i'
      include 'molcul.i'
      include 'moldyn.i'
      include 'cross.i'
      integer i,j,k,init,stop,itx,ity,itz
      real*8 weigh,xmid,ymid,zmid,xfrac,yfrac,zfrac,
     1xcom,ycom,zcom,xaa,yaa,zaa,ucos,usin
      logical lrottt
c
c     locate the center and center of mass of each molecule
c
C$OMP PARALLEL DO PRIVATE(i,j,init,stop,xmid,ymid,zmid,k,weigh,
C$OMP+ zfrac,yfrac,xfrac,xaa,yaa,zaa,itx,ity,itz,ucos,usin,
C$OMP+ xcom,ycom,zcom) DEFAULT(SHARED)
      do 1 i = 1, nmol
      init = imol(1,i)
      stop = imol(2,i)
      xmid = 0.0d0
      ymid = 0.0d0
      zmid = 0.0d0

      do 7 j = init, stop
      k = kmol(j)
      weigh = mass(k)
      xmid = xmid + x(k)*weigh
      ymid = ymid + y(k)*weigh
7     zmid = zmid + z(k)*weigh

      weigh = molmass(i)
      xmid = xmid / weigh
      ymid = ymid / weigh
      zmid = zmid / weigh
c
c     get fractional coordinates of center of mass
c
      if (orthogonal .or. octahedron) then
       zfrac = zmid
       yfrac = ymid
       xfrac = xmid
      else if (monoclinic) then
       zfrac = zmid / beta_sin
       yfrac = ymid
       xfrac = xmid - zfrac*beta_cos
      else if (triclinic) then
       zfrac = zmid / gamma_term
       yfrac = (ymid - zfrac*beta_term) / gamma_sin
       xfrac = xmid - yfrac*gamma_cos - zfrac*beta_cos
      end if
c
      xaa=0.0d0
      yaa=0.0d0
      zaa=0.0d0
c
      if(nper.gt.0)then
       itx=0
       ity=0
       itz=0
       ucos=1.0d0
       usin=0.0d0
c      translate center of mass into the periodic box:
       if(icx.eq.1)then
c       shift-x, along axis:
        call itit(itx,xfrac,xaa,xbox2,xbox)
c       rotate i mass center and all atoms in the molecule:
        if(itx.ne.0)then
         if(.not.lrottt(i,irwp,rwp,itx,init,stop,nper,lper,x,y,z,
     1   ucos,usin,xmid,xbox2,ymid,ybox2,zmid,zbox2,
     2   cco,kmol,cfi,xfrac,yfrac,zfrac,icx,lwall,
     3   mhb,ohb,ihb,xhb,lwdad))xfrac = xmid
        else
c        molecule is inside box, delete it from the list of constraints
C$OMP CRITICAL(cdel)
         call dhb(i,mhb,ohb,ihb,xhb,lwdad)
C$OMP END CRITICAL(cdel)
        endif
        call itit(ity,yfrac,yaa,ybox2,ybox)
        call itit(itz,zfrac,zaa,zbox2,zbox)
       endif

       if(icx.eq.2)then
        call itit(ity,yfrac,yaa,ybox2,ybox)
        if(ity.ne.0)then
         if(.not.lrottt(i,irwp,rwp,ity,init,stop,nper,lper,x,y,z,
     1   ucos,usin,xmid,xbox2,ymid,ybox2,zmid,zbox2,
     2   cco,kmol,cfi,xfrac,yfrac,zfrac,icx,lwall,
     3   mhb,ohb,ihb,xhb,lwdad))yfrac = ymid
        else
C$OMP CRITICAL(cdel2)
         call dhb(i,mhb,ohb,ihb,xhb,lwdad)
C$OMP END CRITICAL(cdel2)
        endif
        call itit(itz,zfrac,zaa,zbox2,zbox)
        call itit(itx,xfrac,xaa,xbox2,xbox)
       endif

       if(icx.eq.3)then
        call itit(itz,zfrac,zaa,zbox2,zbox)
        if(itz.ne.0)then
         if(.not.lrottt(i,irwp,rwp,itz,init,stop,nper,lper,x,y,z,
     1   ucos,usin,xmid,xbox2,ymid,ybox2,zmid,zbox2,
     2   cco,kmol,cfi,xfrac,yfrac,zfrac,icx,lwall,
     3   mhb,ohb,ihb,xhb,lwdad))zfrac = zmid
        else
C$OMP CRITICAL(cdel3)
         call dhb(i,mhb,ohb,ihb,xhb,lwdad)
C$OMP END CRITICAL(cdel3)
        endif
        call itit(itx,xfrac,xaa,xbox2,xbox)
        call itit(ity,yfrac,yaa,ybox2,ybox)
       endif
      else
c     cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       do while (xfrac .gt. xbox2)
        xfrac = xfrac - xbox
       end do
       do while (xfrac .lt. -xbox2)
        xfrac = xfrac + xbox
       end do
       do while (yfrac .gt. ybox2)
        yfrac = yfrac - ybox
       end do
       do while (yfrac .lt. -ybox2)
        yfrac = yfrac + ybox
       end do
       do while (zfrac .gt. zbox2)
        zfrac = zfrac - zbox
       end do
       do while (zfrac .lt. -zbox2)
        zfrac = zfrac + zbox
       end do
      endif
c
c     truncated octahedron needs to have corners removed
      if (octahedron) then
       if (abs(xfrac)+abs(yfrac)+abs(zfrac) .gt. box34) then
        xfrac = xfrac - sign(xbox2,xfrac)
        yfrac = yfrac - sign(ybox2,yfrac)
        zfrac = zfrac - sign(zbox2,zfrac)
       end if
      end if
c
c     convert translated fraction center of mass to Cartesian
      if (orthogonal .or. octahedron) then
       xcom = xfrac
       ycom = yfrac
       zcom = zfrac
      else if (monoclinic) then
       xcom = xfrac + zfrac*beta_cos
       ycom = yfrac
       zcom = zfrac * beta_sin
      else if (triclinic) then
       xcom = xfrac + yfrac*gamma_cos + zfrac*beta_cos
       ycom = yfrac*gamma_sin + zfrac*beta_term
       zcom = zfrac * gamma_term
      end if
c
c     translate coordinates via offset from center of mass
c
      do 2 j = init, stop
      k = kmol(j)
      x(k) = x(k) - xmid + xcom
      y(k) = y(k) - ymid + ycom
2     z(k) = z(k) - zmid + zcom

c     if required, keep the cylinder symmetry
c     rotate velocities and acceleration for this molecule as well:
      if(nper.gt.0)then
       if(icx.eq.1.and.itx.ne.0)then
        do 3 j = init, stop
        k = kmol(j)
        call trb(   v(2,k),   v(3,k),ucos,usin)
        call trb(   a(2,k),   a(3,k),ucos,usin)
3       call trb(aalt(2,k),aalt(3,k),ucos,usin)
       endif
       if(icx.eq.2.and.ity.ne.0)then
        do 4 j = init, stop
        k = kmol(j)
        call trb(   v(3,k),   v(1,k),ucos,usin)
        call trb(   a(3,k),   a(1,k),ucos,usin)
4       call trb(aalt(3,k),aalt(1,k),ucos,usin)
       endif
       if(icx.eq.3.and.itz.ne.0)then
        do 5 j = init, stop
        k = kmol(j)
        call trb(   v(1,k),   v(2,k),ucos,usin)
        call trb(   a(1,k),   a(2,k),ucos,usin)
5       call trb(aalt(1,k),aalt(2,k),ucos,usin)
       endif
      endif
1     continue

      return
      end

      subroutine dhb(i,mhb,ohb,ihb,xhb,lwdad)
      implicit none
      integer i,mhb(*),xhb(*),ihb,j,k   
      logical lwdad
      real*8 ohb(*)
      do 1 j=1,ihb
1     if(mhb(j).eq.i)goto 2
      return
2     do 3 k=j,ihb-1
      mhb(k)=mhb(k+1)
      ohb(k)=ohb(k+1)
3     xhb(k)=xhb(k+1)
      ihb=ihb-1
      if(lwdad)write(6,*)i,' deleted from constraints d#d'
      return
      end
