c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     #################################################################
c     ##                                                             ##
c     ##  subroutine eangle1  --  angle bend energy and derivatives  ##
c     ##                                                             ##
c     #################################################################
c
c
c     "eangle1" calculates the angle bending potential energy and
c     the first derivatives with respect to Cartesian coordinates;
c     projected in-plane angles at trigonal centers, special linear
c     or Fourier angle bending terms are optionally used
c
c
      subroutine eangle1
      implicit none
      include 'sizes.i'
      include 'angle.i'
      include 'angpot.i'
      include 'atoms.i'
      include 'bound.i'
      include 'deriv.i'
      include 'energi.i'
      include 'group.i'
      include 'math.i'
      include 'usage.i'
      include 'virial.i'
      include 'cross.i'
      integer i,ia,ib,ic,id
      real*8 e,ideal,force,fold,factor,dot,cosine,sine,angle,fgrp,dt,
     1dt2,dt3,dt4,deddt,term,terma,termc,xia,yia,zia,xib,yib,zib,xic,
     2yic,zic,xid,yid,zid,xab,yab,zab,xcb,ycb,zcb,xp,yp,zp,rp,xad,yad,
     3zad,xbd,ybd,zbd,xdb,ydb,zdb,xcd,ycd,zcd,xip,yip,zip,xap,yap,zap,
     4xcp,ycp,zcp,rab2,rcb2,rap2,rcp2,xt,yt,zt,rt2,ptrt2,xm,ym,zm,rm,
     5delta,delta2,dedxia,dedyia,dedzia,dedxib,dedyib,dedzib,dedxic,
     6dedyic,dedzic,dedxid,dedyid,dedzid,dedxip,dedyip,dedzip,dpdxia,
     7dpdyia,dpdzia,dpdxic,dpdyic,dpdzic,vxx,vyy,vzz,vyx,vzx,vzy, 
     8ucosc,usinc,ucosa,usina,ucosp,usinp,xpb,ucosd,usind,
     9ypb,zpb,xtt,ytt,ztt
      integer itxa,itya,itza,itxc,ityc,itzc,itxp,ityp,itzp,
     1itxd,ityd,itzd
      logical proceed,lrota,lrotc,lrotd,lrotp

      include 'parallel.i'
      include 'cfast.i'
      integer*4 ix,iy,j,i3,nsl(maxproc),nel(maxproc)
      real*8 e_p(maxproc),vir_p(maxproc,3,3),
     1m_p(maxproc),m_m(maxproc)

      real*8,allocatable::g_p(:,:)

      allocate(g_p(nproc,3*n))
c
      call ppd(nproc,maxproc,nsl,nel,nangle)
C$OMP PARALLEL DO PRIVATE(i,j,ix,iy) DEFAULT(SHARED)
      do 11 i=1,nproc
      e_p(i)=0.0d0
      m_p(i)=0.0d0
      m_m(i)=0.0d0
      do 111 ix=1,3
      do 111 iy=1,3
111   vir_p(i,ix,iy)=0.0d0
      do 11 j=1,3*n
11    g_p(i,j)=0.0d0
c
c     zero out energy and first derivative components
      ea = 0.0d0
C$OMP PARALLEL DO PRIVATE(i) DEFAULT(SHARED)
      do 1 i = 1, n
      dea(1,i) = 0.0d0
      dea(2,i) = 0.0d0
1     dea(3,i) = 0.0d0
c
c     calculate the bond angle bending energy term
c
C$OMP PARALLEL PRIVATE(ipr,i,ia,ib,ic,id,ideal,force,proceed,
C$OMP+ xia,yia,zia, xib,yib,zib, xic,yic,zic, 
C$OMP+ xab,yab,zab,xcb, ycb,zcb, xtt,ytt,ztt,ucosa,usina,
C$OMP+ itxa,itya,itza,ucosc,usinc,itxc,ityc,itzc,lrota,lrotc,
C$OMP+ rab2,rcb2,xp,yp,zp,rp,dot,cosine,angle,dt,dt2,dt3,dt4,e,deddt,
C$OMP+ factor,sine,fold,terma,termc,dedxia,dedyia,dedzia,dedxic,
C$OMP+ dedyic,dedzic,dedxib,dedyib,dedzib,xid,yid,zid,xdb,ydb,zdb,
C$OMP+ xbd,ybd,zbd,xcd,ycd,zcd,xad,yad,zad,ucosd,usind,itxd,ityd,itzd,
C$OMP+ lrotd,xt,yt,zt,rt2,delta,xip,yip,zip,xpb,ypb,zpb,xap,yap,zap,
C$OMP+ xcp,ycp,zcp,ucosp,usinp,itxp,ityp,itzp,lrotp,rap2,rcp2,xm,ym,
C$OMP+ zm,rm,dedxip,dedyip,dedzip,delta2,ptrt2,term,dpdxia,dpdyia,
C$OMP+ dpdzia,dpdxic,dpdyic,dpdzic,dedxid,dedyid,dedzid,
C$OMP+ vxx,vyx,vzx,vyy,vzy,vzz) DEFAULT(SHARED)
      ipr=OMP_GET_THREAD_NUM()+1
      do 101 i=nsl(ipr),nel(ipr)
      ia = iang(1,i)
      ib = iang(2,i)
      ic = iang(3,i)
      id = iang(4,i)
      ideal = anat(i)
      force = ak(i)
c
c     decide whether to compute the current interaction
      proceed = .true.
      if (angtyp(i) .eq. 'IN-PLANE') then
       if (use_group)  call groups (proceed,fgrp,ia,ib,ic,id,0,0)
       if (proceed)  proceed = (use(ia) .or. use(ib) .or.
     &                               use(ic) .or. use(id))
      else
       if (use_group)  call groups (proceed,fgrp,ia,ib,ic,0,0,0)
       if (proceed)  proceed = (use(ia) .or. use(ib) .or. use(ic))
      end if
c
      if (proceed) then
c      get the coordinates of the atoms in the angle
       call ass3(xia,x(ia),yia,y(ia),zia,z(ia))
       call ass3(xib,x(ib),yib,y(ib),zib,z(ib))
       call ass3(xic,x(ic),yic,y(ic),zic,z(ic))
c
c      compute the bond angle bending energy and gradient
c      write(6,*)i,ia,ib,ic,id,angtyp(i)
       if (angtyp(i) .ne. 'IN-PLANE') then
        call ass3(xab,xia-xib,yab,yia-yib,zab,zia-zib)
        call ass3(xcb,xic-xib,ycb,yic-yib,zcb,zic-zib)

        if(nper.ne.0)then
c        rotate position of a and c according to b:
c        (cannot just rotate xab etc)
c        1) find translation/roattion:
         call ass3(xtt,x(ia),ytt,y(ia),ztt,z(ia))
         call imaget(xab,yab,zab,ia,ucosa,usina,itxa,itya,itza,
     1   xtt,ytt,ztt,xib,yib,zib)
         call ass3(xtt,x(ic),ytt,y(ic),ztt,z(ic))
         call imaget(xcb,ycb,zcb,ic,ucosc,usinc,itxc,ityc,itzc,
     1   xtt,ytt,ztt,xib,yib,zib)
c        2) new positions and distances
         lrota=(icx.eq.1.and.itxa.ne.0).or.
     1         (icx.eq.2.and.itya.ne.0).or.
     1         (icx.eq.3.and.itza.ne.0)
         lrotc=(icx.eq.1.and.itxc.ne.0).or.
     1         (icx.eq.2.and.ityc.ne.0).or.
     1         (icx.eq.3.and.itzc.ne.0)
         if(lrota)then
          call newpos(icx,itxa,itya,itza,xia,yia,zia,usina,ucosa)
          call ass3(xab,xia-xib,yab,yia-yib,zab,zia-zib)
         endif
         if(lrotc)then
          call newpos(icx,itxc,ityc,itzc,xic,yic,zic,usinc,ucosc)
          call ass3(xcb,xic-xib,ycb,yic-yib,zcb,zic-zib)
         endif
        else
         if (use_polymer) then
          call image (xab,yab,zab)
          call image (xcb,ycb,zcb)
         endif
        end if
        if(ia.eq.3.and.ib.eq.2.and.ic.eq.134)then
c        write(6,6001)ia,ib,ic,x(ia),y(ia),z(ia),x(ib),y(ib),z(ib),
c    1   x(ic),y(ic),z(ic),xia,yia,zia,xib,yib,zib,xic,yic,zic,
c    2   usina,ucosa,usinc,ucosc,itxa,itya,itza,itxc,ityc,itzc
c001     format(' ia ib ic:',3i5,/,' xa:',3f12.6,/,' xb:',3f12.6,/,
c    1   ' xc:',3f12.6,/,' ap:',3f12.6,/,' bp:',3f12.6,/,
c    2   ' cp:',3f12.6,/,' sina cosa sinc cosc:',4f10.6,/,
c    3   ' itxa ityz itza itxc ityc itzc:',6i2)
c        stop
        endif
 
        rab2 = xab*xab + yab*yab + zab*zab
        rcb2 = xcb*xcb + ycb*ycb + zcb*zcb
c       rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
        if (rab2.ne.0.0d0 .and. rcb2.ne.0.0d0) then
         xp = ycb*zab - zcb*yab
         yp = zcb*xab - xcb*zab
         zp = xcb*yab - ycb*xab
         rp = sqrt(xp*xp + yp*yp + zp*zp)
         rp = max(rp,0.000001d0)
         dot = xab*xcb + yab*ycb + zab*zcb
         cosine = dot / sqrt(rab2*rcb2)
         cosine = min(1.0d0,max(-1.0d0,cosine))
         angle = radian * acos(cosine)
c        write(6,*)angle,ideal
c
c        get the energy and master chain rule term for derivatives
c
         if (angtyp(i) .eq. 'HARMONIC') then
          dt = angle - ideal
          dt2 = dt * dt
          dt3 = dt2 * dt
          dt4 = dt2 * dt2
          e = angunit * force * dt2
     &    * (1.0d0+cang*dt+qang*dt2+pang*dt3+sang*dt4)
          deddt = angunit * force * dt * radian
     &    * (2.0d0 + 3.0d0*cang*dt + 4.0d0*qang*dt2
     &    + 5.0d0*pang*dt3 + 6.0d0*sang*dt4)
         else if (angtyp(i) .eq. 'LINEAR') then
          factor = 2.0d0 * angunit * radian**2
          sine = sqrt(1.0d0-cosine*cosine)
          e = factor * force * (1.0d0+cosine)
          deddt = -factor * force * sine
         else if (angtyp(i) .eq. 'FOURIER') then
          fold = afld(i)
          factor = 2.0d0 * angunit * (radian/fold)**2
          cosine = cos((fold*angle-ideal)/radian)
          sine = sin((fold*angle-ideal)/radian)
          e = factor * force * (1.0d0+cosine)
          deddt = -factor * force * fold * sine
         end if
c
c        scale the interaction based on its group membership
         if (use_group) then
          e = e * fgrp
          deddt = deddt * fgrp
         end if
c
c        compute derivative components for this interaction
         terma = -deddt / (rab2*rp)
         termc = deddt / (rcb2*rp)
         dedxia = terma * (yab*zp-zab*yp)
         dedyia = terma * (zab*xp-xab*zp)
         dedzia = terma * (xab*yp-yab*xp)
         dedxic = termc * (ycb*zp-zcb*yp)
         dedyic = termc * (zcb*xp-xcb*zp)
         dedzic = termc * (xcb*yp-ycb*xp)
         dedxib = -dedxia - dedxic
         dedyib = -dedyia - dedyic
         dedzib = -dedzia - dedzic
c
c        ia,ic  rotate back if rotated
         if(nper.ne.0)then
          if(icx.eq.1)then
           if(itxa.ne.0)call trb(dedyia,dedzia,ucosa,-usina)
           if(itxc.ne.0)call trb(dedyic,dedzic,ucosc,-usinc)
          endif
          if(icx.eq.2)then
           if(itya.ne.0)call trb(dedzia,dedxia,ucosa,-usina)
           if(ityc.ne.0)call trb(dedzic,dedxic,ucosc,-usinc)
          endif
          if(icx.eq.3)then
           if(itza.ne.0)call trb(dedxia,dedyia,ucosa,-usina)
           if(itzc.ne.0)call trb(dedxic,dedyic,ucosc,-usinc)
          endif               
          call incrtwistp(m_p(ipr),m_m(ipr),
     1    dedxia,dedyia,dedzia,x(ia),y(ia),z(ia),icx)
          call incrtwistp(m_p(ipr),m_m(ipr),
     1    dedxib,dedyib,dedzib,x(ib),y(ib),z(ib),icx)
          call incrtwistp(m_p(ipr),m_m(ipr),
     1    dedxic,dedyic,dedzic,x(ic),y(ic),z(ic),icx)
         endif

c        increment the total torsional angle energy and gradient
         e_p(ipr) = e_p(ipr) + e
         i3=3*(ia-1)
         g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxia
         g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyia
         g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzia
         i3=3*(ib-1)
         g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxib
         g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyib
         g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzib
         i3=3*(ic-1)
         g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxic
         g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyic
         g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzic

c        increment the internal virial tensor components
         vxx = xab*dedxia + xcb*dedxic
         vyx = yab*dedxia + ycb*dedxic
         vzx = zab*dedxia + zcb*dedxic
         vyy = yab*dedyia + ycb*dedyic
         vzy = zab*dedyia + zcb*dedyic
         vzz = zab*dedzia + zcb*dedzic
         call incrvirp(maxproc,ipr,vir_p,vxx,vyx,vzx,vyy,vzy,vzz)
        end if
c       rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr
c
c      compute the projected in-plane angle energy and gradient
       else
c      in plane
        call ass3(xid,x(id),yid,y(id),zid,z(id))
        call ass3(xdb,xid-xib,ydb,yid-yib,zdb,zid-zib)
        call ass3(xad,xia-xid,yad,yia-yid,zad,zia-zid)
        call ass3(xbd,xib-xid,ybd,yib-yid,zbd,zib-zid)
        call ass3(xcd,xic-xid,ycd,yic-yid,zcd,zic-zid)
        if(nper.ne.0)then
         call ass3(xtt,x(ia),ytt,y(ia),ztt,z(ia))
         call imaget(xab,yab,zab,ia,ucosa,usina,itxa,itya,itza,
     1   xtt,ytt,ztt,xib,yib,zib)
         call ass3(xtt,x(ic),ytt,y(ic),ztt,z(ic))
         call imaget(xcb,ycb,zcb,ic,ucosc,usinc,itxc,ityc,itzc,
     1   xtt,ytt,ztt,xib,yib,zib)
         call ass3(xtt,x(id),ytt,y(id),ztt,z(id))
         call imaget(xdb,ydb,zdb,id,ucosd,usind,itxd,ityd,itzd,
     1   xtt,ytt,ztt,xib,yib,zib)
         lrota=(icx.eq.1.and.itxa.ne.0).or.
     1         (icx.eq.2.and.itya.ne.0).or.
     1         (icx.eq.3.and.itza.ne.0)
         lrotc=(icx.eq.1.and.itxc.ne.0).or.
     1         (icx.eq.2.and.ityc.ne.0).or.
     1         (icx.eq.3.and.itzc.ne.0)
         lrotd=(icx.eq.1.and.itxd.ne.0).or.
     1         (icx.eq.2.and.ityd.ne.0).or.
     1         (icx.eq.3.and.itzd.ne.0)
         if(lrota)then
          call newpos(icx,itxa,itya,itza,xia,yia,zia,usina,ucosa)
         endif
         if(lrotc)then
          call newpos(icx,itxc,ityc,itzc,xic,yic,zic,usinc,ucosc)
         endif
         if(lrotd)then
          call newpos(icx,itxd,ityd,itzd,xid,yid,zid,usind,ucosd)
         endif
         call ass3(xad,xia-xid,yad,yia-yid,zad,zia-zid)
         call ass3(xbd,xib-xid,ybd,yib-yid,zbd,zib-zid)
         call ass3(xcd,xic-xid,ycd,yic-yid,zcd,zic-zid)
        else
         if (use_polymer) then
          call image (xad,yad,zad)
          call image (xbd,ybd,zbd)
          call image (xcd,ycd,zcd)
         endif
        end if
        xt = yad*zcd - zad*ycd
        yt = zad*xcd - xad*zcd
        zt = xad*ycd - yad*xcd
        rt2 = xt*xt + yt*yt + zt*zt
        delta = -(xt*xbd + yt*ybd + zt*zbd) / rt2
        xip = xib + xt*delta
        yip = yib + yt*delta
        zip = zib + zt*delta
        call ass3( xpb,xip-xib,ypb,yip-yib,zpb,zip-zib)
        call ass3( xap,xia-xip,yap,yia-yip,zap,zia-zip)
        call ass3( xcp,xic-xip,ycp,yic-yip,zcp,zic-zip)
        if(nper.ne.0)then
c        relate p to b, supply ia as molecule identificator only:
         call ass3(xtt,xip,ytt,yip,ztt,zip)
         call imaget(xpb,ypb,zpb,ia,ucosp,usinp,itxp,ityp,itzp,
     1   xtt,ytt,ztt,xib,yib,zib)
         lrotp=(icx.eq.1.and.itxp.ne.0).or.
     1         (icx.eq.2.and.ityp.ne.0).or.
     1         (icx.eq.3.and.itzp.ne.0)
         if(lrotp)then
          call newpos(icx,itxp,ityp,itzp,xip,yip,zip,usinp,ucosp)
          call ass3( xap,xia-xip,yap,yia-yip,zap,zia-zip)
          call ass3( xcp,xic-xip,ycp,yic-yip,zcp,zic-zip)
         endif
        else
         if (use_polymer) then
          call image (xap,yap,zap)
          call image (xcp,ycp,zcp)
         endif
        endif
        rap2 = xap*xap + yap*yap + zap*zap
        rcp2 = xcp*xcp + ycp*ycp + zcp*zcp
c       RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
        if (rap2.ne.0.0d0 .and. rcp2.ne.0.0d0) then
         xm = ycp*zap - zcp*yap
         ym = zcp*xap - xcp*zap
         zm = xcp*yap - ycp*xap
         rm = sqrt(xm*xm + ym*ym + zm*zm)
         rm = max(rm,0.000001d0)
         dot = xap*xcp + yap*ycp + zap*zcp
         cosine = dot / sqrt(rap2*rcp2)
         cosine = min(1.0d0,max(-1.0d0,cosine))
         angle = radian * acos(cosine)
c
c        get the energy and master chain rule term for derivatives
         dt = angle - ideal
         dt2 = dt * dt
         dt3 = dt2 * dt
         dt4 = dt2 * dt2
         e = angunit * force * dt2
     &         * (1.0d0+cang*dt+qang*dt2+pang*dt3+sang*dt4)
         deddt = angunit * force * dt * radian
     &            * (2.0d0 + 3.0d0*cang*dt + 4.0d0*qang*dt2
     &                 + 5.0d0*pang*dt3 + 6.0d0*sang*dt4)
c
c        scale the interaction based on its group membership
         if (use_group) then
          e = e * fgrp
          deddt = deddt * fgrp
         end if
c
c        chain rule terms for first derivative components
         terma = -deddt / (rap2*rm)
         termc = deddt / (rcp2*rm)
         dedxia = terma * (yap*zm-zap*ym)
         dedyia = terma * (zap*xm-xap*zm)
         dedzia = terma * (xap*ym-yap*xm)
         dedxic = termc * (ycp*zm-zcp*ym)
         dedyic = termc * (zcp*xm-xcp*zm)
         dedzic = termc * (xcp*ym-ycp*xm)
         dedxip = -dedxia - dedxic
         dedyip = -dedyia - dedyic
         dedzip = -dedzia - dedzic
c
c        chain rule components for the projection of the central atom
         delta2 = 2.0d0 * delta
         ptrt2 = (dedxip*xt + dedyip*yt + dedzip*zt) / rt2
         term = (zcd*ybd-ycd*zbd) + delta2*(yt*zcd-zt*ycd)
         dpdxia = delta*(ycd*dedzip-zcd*dedyip) + term*ptrt2
         term = (xcd*zbd-zcd*xbd) + delta2*(zt*xcd-xt*zcd)
         dpdyia = delta*(zcd*dedxip-xcd*dedzip) + term*ptrt2
         term = (ycd*xbd-xcd*ybd) + delta2*(xt*ycd-yt*xcd)
         dpdzia = delta*(xcd*dedyip-ycd*dedxip) + term*ptrt2
         term = (yad*zbd-zad*ybd) + delta2*(zt*yad-yt*zad)
         dpdxic = delta*(zad*dedyip-yad*dedzip) + term*ptrt2
         term = (zad*xbd-xad*zbd) + delta2*(xt*zad-zt*xad)
         dpdyic = delta*(xad*dedzip-zad*dedxip) + term*ptrt2
         term = (xad*ybd-yad*xbd) + delta2*(yt*xad-xt*yad)
         dpdzic = delta*(yad*dedxip-xad*dedyip) + term*ptrt2
c
c        compute derivative components for this interaction
         dedxia = dedxia + dpdxia
         dedyia = dedyia + dpdyia
         dedzia = dedzia + dpdzia
         dedxib = dedxip
         dedyib = dedyip
         dedzib = dedzip
         dedxic = dedxic + dpdxic
         dedyic = dedyic + dpdyic
         dedzic = dedzic + dpdzic
         dedxid = -dedxia - dedxib - dedxic
         dedyid = -dedyia - dedyib - dedyic
         dedzid = -dedzia - dedzib - dedzic

c        ia,ic,id  rotate back if rotated
         if(nper.ne.0)then
          if(icx.eq.1)then
           if(itxa.ne.0)call trb(dedyia,dedzia,ucosa,-usina)
           if(itxc.ne.0)call trb(dedyic,dedzic,ucosc,-usinc)
           if(itxd.ne.0)call trb(dedyid,dedzid,ucosd,-usind)
          endif
          if(icx.eq.2)then
           if(itya.ne.0)call trb(dedzia,dedxia,ucosa,-usina)
           if(ityc.ne.0)call trb(dedzic,dedxic,ucosc,-usinc)
           if(ityd.ne.0)call trb(dedzid,dedxid,ucosd,-usind)
          endif
          if(icx.eq.3)then
           if(itza.ne.0)call trb(dedxia,dedyia,ucosa,-usina)
           if(itzc.ne.0)call trb(dedxic,dedyic,ucosc,-usinc)
           if(itzd.ne.0)call trb(dedxid,dedyid,ucosd,-usind)
          endif               
          call incrtwistp(m_p(ipr),m_m(ipr),
     1    dedxia,dedyia,dedzia,x(ia),y(ia),z(ia),icx)
          call incrtwistp(m_p(ipr),m_m(ipr),
     1    dedxib,dedyib,dedzib,x(ib),y(ib),z(ib),icx)
          call incrtwistp(m_p(ipr),m_m(ipr),
     1    dedxic,dedyic,dedzic,x(ic),y(ic),z(ic),icx)
          call incrtwistp(m_p(ipr),m_m(ipr),
     1    dedxid,dedyid,dedzid,x(id),y(id),z(id),icx)
         endif

c        increment the total torsional angle energy and gradient
         e_p(ipr) = e_p(ipr) + e
         i3=3*(ia-1)
         g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxia
         g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyia
         g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzia
         i3=3*(ib-1)
         g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxib
         g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyib
         g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzib
         i3=3*(ic-1)
         g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxic
         g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyic
         g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzic
         i3=3*(id-1)
         g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxid
         g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyid
         g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzid

c        increment the internal virial tensor components
         vxx = xad*dedxia + xbd*dedxib + xcd*dedxic
         vyx = yad*dedxia + ybd*dedxib + ycd*dedxic
         vzx = zad*dedxia + zbd*dedxib + zcd*dedxic
         vyy = yad*dedyia + ybd*dedyib + ycd*dedyic
         vzy = zad*dedyia + zbd*dedyib + zcd*dedyic
         vzz = zad*dedzia + zbd*dedzib + zcd*dedzic
         call incrvirp(maxproc,ipr,vir_p,vxx,vyx,vzx,vyy,vzy,vzz)
        end if
c       RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
       end if
c      in plane
      end if
c     proceed
101   continue 
C$OMP END PARALLEL

C$OMP PARALLEL DO PRIVATE(i,ipr,i3) DEFAULT(SHARED)
      do 202 i=1,n
      i3=3*(i-1)
      do 202 ipr=1,nproc
      dea(1,i)=dea(1,i)+g_p(ipr,i3+1)
      dea(2,i)=dea(2,i)+g_p(ipr,i3+2)
202   dea(3,i)=dea(3,i)+g_p(ipr,i3+3)

      do 200 ipr=1,nproc
      vir(1,1)=vir(1,1)+vir_p(ipr,1,1)
      vir(1,2)=vir(1,2)+vir_p(ipr,1,2)
      vir(1,3)=vir(1,3)+vir_p(ipr,1,3)
      vir(2,1)=vir(2,1)+vir_p(ipr,2,1)
      vir(2,2)=vir(2,2)+vir_p(ipr,2,2)
      vir(2,3)=vir(2,3)+vir_p(ipr,2,3)
      vir(3,1)=vir(3,1)+vir_p(ipr,3,1)
      vir(3,2)=vir(3,2)+vir_p(ipr,3,2)
      vir(3,3)=vir(3,3)+vir_p(ipr,3,3)
      twm=twm+m_m(ipr)
      twp=twp+m_p(ipr)
200   ea=ea+e_p(ipr)

      return
      end

