c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine ebond1  --  bond stretch energy & derivatives  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "ebond1" calculates the bond stretching energy and
c     first derivatives with respect to Cartesian coordinates
c
c
      subroutine ebond1
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bndpot.i'
      include 'bond.i'
      include 'bound.i'
      include 'deriv.i'
      include 'energi.i'
      include 'group.i'
      include 'usage.i'
      include 'virial.i'
      include 'cross.i'
c     include 'rotc.i'
c     
c     itx,ity,itz ... numbr of rotations/translations 
c     ucos,usin ... the rotational matrix elements
c     f .. the rotational damping factor
      integer itx,ity,itz
      real*8 ucos,usin,frt
      common /rotcc/frt,ucos,usin,itx,ity,itz
      integer i,ia,ib
      real*8 e,ideal,force,xt,yt,zt
      real*8 expterm,bde,fgrp
      real*8 dt,dt2,deddt
      real*8 de,dedx,dedy,dedz
      real*8 xab,yab,zab,rab,xia,yia,zia,xib,yib,zib
      real*8 vxx,vyy,vzz,usina,ucosa
      real*8 vyx,vzx,vzy
      logical proceed,lrot

      include 'parallel.i'
      include 'cfast.i'
      integer*4 ix,iy,j,i3,nsl(maxproc),nel(maxproc)
      real*8 e_p(maxproc),vir_p(maxproc,3,3),
     1m_p(maxproc),m_m(maxproc)

      real*8,allocatable::g_p(:,:)

      allocate(g_p(nproc,3*n))

      call ppd(nproc,maxproc,nsl,nel,nbond)
C$OMP PARALLEL DO PRIVATE(i,j,ix,iy) DEFAULT(SHARED)
      do 11 i=1,nproc
      e_p(i)=0.0d0
      m_p(i)=0.0d0
      m_m(i)=0.0d0
      do 111 ix=1,3
      do 111 iy=1,3
111   vir_p(i,ix,iy)=0.0d0
      do 11 j=1,3*n
11    g_p(i,j)=0.0d0
c
c     zero out the bond energy and first derivatives
      eb = 0.0d0
C$OMP PARALLEL DO PRIVATE(i) DEFAULT(SHARED)
      do 1 i = 1, n
      deb(1,i) = 0.0d0
      deb(2,i) = 0.0d0
1     deb(3,i) = 0.0d0
c
c     calculate the bond stretch energy and first derivatives
c
C$OMP PARALLEL PRIVATE(i,ia,ib,ideal,force,proceed,xia,yia,zia,
C$OMP+ xt,yt,zt,xib,yib,zib,xab,yab,zab,ucosa,usina,itx,ity,itz,
C$OMP+ lrot,rab,dt,dt2,e,deddt,expterm,bde,de,dedx,dedy,dedz,eb,vxx,vyx,
C$OMP+ vzx,vyy,vzy,vzz,ipr) DEFAULT(SHARED)
      ipr=OMP_GET_THREAD_NUM()+1
      do 101 i=nsl(ipr),nel(ipr)
      ia = ibnd(1,i)
      ib = ibnd(2,i)
      ideal = bl(i)
      force = bk(i)
c
c     decide whether to compute the current interaction
      proceed = .true.
      if (use_group)  call groups (proceed,fgrp,ia,ib,0,0,0,0)
      if (proceed)  proceed = (use(ia) .or. use(ib))
c
c     compute the value of the bond length deviation
      if (proceed) then
       call ass3(xia,x(ia),yia,y(ia),zia,z(ia))
       call ass3(xib,x(ib),yib,y(ib),zib,z(ib))
       call ass3(xab,xia-xib,yab,yia-yib,zab,zia-zib)
c
       if(nper.ne.0)then
        call ass3(xt,xia,yt,yia,zt,zia)
        call imaget(xab,yab,zab,ia,ucosa,usina,itx,ity,itz,
     1  xt,yt,zt,xib,yib,zib)
        lrot=(icx.eq.1.and.itx.ne.0).or.
     1       (icx.eq.2.and.ity.ne.0).or.
     1       (icx.eq.3.and.itz.ne.0)
        if(lrot)then
         call newpos(icx,itx,ity,itz,xia,yia,zia,usina,ucosa)
         call ass3(xab,xia-xib,yab,yia-yib,zab,zia-zib)
        endif
       else
        if (use_polymer)then
         call image (xab,yab,zab)
        endif
       endif
       rab = sqrt(xab*xab + yab*yab + zab*zab)
       dt = rab - ideal
c      if((ia.eq.2.and.ib.eq.134).or.(ia.eq.22.and.ib.eq.23))then
c       write(6,*)ia,ib,ideal,rab
c       write(6,*)'ra',x(ia),y(ia),z(ia)
c       write(6,*)'rb',xib,yib,zib
c       write(6,*)'rap',xia,yia,zia
c       write(6,*)'rabp',xab,yab,zab
c       write(6,*)'itx,itxy itz ucosa,usina',itx,ity,itz,ucosa,usina
c       if(ib.eq.23)stop
c      endif
c
c      harmonic potential uses Taylor expansion of Morse potential
c      through the fourth power of the bond length deviation
       if (bndtyp .eq. 'HARMONIC') then
        dt2 = dt * dt
        e = bndunit * force * dt2 * (1.0d0+cbnd*dt+qbnd*dt2)
        deddt = 2.0d0 * bndunit * force * dt
     &  * (1.0d0+1.5d0*cbnd*dt+2.0d0*qbnd*dt2)
c
c      Morse potential uses energy = BDE * (1 - e**(-alpha*dt))**2)
c      with the approximations alpha = sqrt(ForceConst/BDE) = -2
c      and BDE = Bond Dissociation Energy = ForceConst/alpha**2
       else if (bndtyp .eq. 'MORSE') then
        expterm = exp(-2.0d0*dt)
        bde = 0.25d0 * bndunit * force
        e = bde * (1.0d0-expterm)**2
        deddt = 4.0d0 * bde * (1.0d0-expterm) * expterm
       end if
c
c      scale the interaction based on its group membership
       if (use_group) then
        e = e * fgrp
        deddt = deddt * fgrp
       end if
c
c      compute chain rule terms needed for derivatives
       if (rab .eq. 0.0d0) then
        de = 0.0d0
       else
        de = deddt / rab
       end if
       dedx = de * xab
       dedy = de * yab
       dedz = de * zab
c
       i3=3*(ib-1)
       g_p(ipr,i3+1)=g_p(ipr,i3+1)- dedx
       g_p(ipr,i3+2)=g_p(ipr,i3+2)- dedy
       g_p(ipr,i3+3)=g_p(ipr,i3+3)- dedz

c      if atom ia was rotated by imagep, rotate gradient back
       if(nper.ne.0)then
         call incrtwistp(m_p(ipr),m_m(ipr),
     1   -dedx,-dedy,-dedz,x(ib),y(ib),z(ib),icx)
        if(icx.eq.1.and.itx.ne.0)call trb(dedy,dedz,ucosa,-usina)
        if(icx.eq.2.and.ity.ne.0)call trb(dedz,dedx,ucosa,-usina)
        if(icx.eq.3.and.itz.ne.0)call trb(dedx,dedy,ucosa,-usina)
        call incrtwistp(m_p(ipr),m_m(ipr),
     1  dedx,dedy,dedz,x(ia),y(ia),z(ia),icx)
       endif

       e_p(ipr) = e_p(ipr) + e
       i3=3*(ia-1)
       g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedx
       g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedy
       g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedz

       vxx = xab * dedx
       vyx = yab * dedx
       vzx = zab * dedx
       vyy = yab * dedy
       vzy = zab * dedy
       vzz = zab * dedz
       call incrvirp(maxproc,ipr,vir_p,vxx,vyx,vzx,vyy,vzy,vzz)
      end if

101   continue 
C$OMP END PARALLEL

C$OMP PARALLEL DO PRIVATE(i,ipr,i3) DEFAULT(SHARED)
      do 202 i=1,n
      i3=3*(i-1)
      do 202 ipr=1,nproc
      deb(1,i)=deb(1,i)+g_p(ipr,i3+1)
      deb(2,i)=deb(2,i)+g_p(ipr,i3+2)
202   deb(3,i)=deb(3,i)+g_p(ipr,i3+3)

      do 200 ipr=1,nproc
      vir(1,1)=vir(1,1)+vir_p(ipr,1,1)
      vir(1,2)=vir(1,2)+vir_p(ipr,1,2)
      vir(1,3)=vir(1,3)+vir_p(ipr,1,3)
      vir(2,1)=vir(2,1)+vir_p(ipr,2,1)
      vir(2,2)=vir(2,2)+vir_p(ipr,2,2)
      vir(2,3)=vir(2,3)+vir_p(ipr,2,3)
      vir(3,1)=vir(3,1)+vir_p(ipr,3,1)
      vir(3,2)=vir(3,2)+vir_p(ipr,3,2)
      vir(3,3)=vir(3,3)+vir_p(ipr,3,3)
      twm=twm+m_m(ipr)
      twp=twp+m_p(ipr)
200   eb=eb+e_p(ipr)

      return
      end
