c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1991  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine eimptor1  --  impr. torsion energy & gradient  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "eimptor1" calculates improper torsion energy and its
c     first derivatives with respect to Cartesian coordinates
c
c
      subroutine eimptor1
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bound.i'
      include 'deriv.i'
      include 'energi.i'
      include 'group.i'
      include 'imptor.i'
      include 'torpot.i'
      include 'usage.i'
      include 'virial.i'
      include 'cross.i'
      include 'cell.i'
      integer i,ia,ib,ic,id,itxd,ityd,itzd,itxa,itya,itza,itxc,ityc,itzc
      real*8 e,dedphi,rcb,fgrp,xt,yt,zt,xu,yu,zu,xtu,ytu,ztu,rt2,ru2,
     1rtru,v1,v2,v3,c1,c2,c3,s1,s2,s3,sine,cosine,sine2,cosine2,sine3,
     2cosine3,phi1,phi2,phi3,xia,yia,zia,xib,yib,zib,xic,yic,zic,xid,
     3yid,zid,xba,yba,zba,xab,yab,zab,xcb,ycb,zcb,xdb,ydb,zdb,xdc,ydc,
     4zdc,xca,yca,zca,dphi1,dphi2,dphi3,dedxt,dedyt,dedzt,dedxu,dedyu,
     5dedzu,dedxia,dedyia,dedzia,dedxib,dedyib,dedzib,dedxic,dedyic,
     6dedzic,dedxid,dedyid,dedzid,vxx,vyy,vzz,vyx,vzx,vzy,ucosd,
     7usind,ucosa,usina,ucosc,usinc,xtt,ytt,ztt
      logical proceed,lrota,lrotc,lrotd

      include 'parallel.i'
      include 'cfast.i'
      integer*4 ix,iy,j,i3,nsl(maxproc),nel(maxproc)
      real*8 e_p(maxproc),vir_p(maxproc,3,3),
     1m_p(maxproc),m_m(maxproc)
      real*8,allocatable::g_p(:,:)

      allocate(g_p(nproc,3*n))

c
      call ppd(nproc,maxproc,nsl,nel,nitors)
C$OMP PARALLEL DO PRIVATE(i,j,ix,iy) DEFAULT(SHARED)
      do 11 i=1,nproc
      e_p(i)=0.0d0
      m_p(i)=0.0d0
      m_m(i)=0.0d0
      do 111 ix=1,3
      do 111 iy=1,3
111   vir_p(i,ix,iy)=0.0d0
      do 11 j=1,3*n
11    g_p(i,j)=0.0d0
c
c     zero out the torsional energy and first derivatives
c
      eit = 0.0d0
C$OMP PARALLEL DO PRIVATE(i) DEFAULT(SHARED)
      do 1 i = 1, n
      deit(1,i) = 0.0d0
      deit(2,i) = 0.0d0
1     deit(3,i) = 0.0d0
c
c     calculate the improper torsional angle energy term

C$OMP  PARALLEL PRIVATE(i,ia,ib,ic,id,proceed,
C$OMP+ xia,yia,zia, xib,yib,zib, xic,yic,zic, xid,yid,zid,
C$OMP+ xba,xcb,xdc,xab,xca,xdb,
C$OMP+ yba,ycb,ydc,yab,yca,ydb,
C$OMP+ zba,zcb,zdc,zab,zca,zdb,
C$OMP+ xtt,ytt,ztt,
C$OMP+ itxa,itya,itza, itxc,ityc,itzc, itxd,ityd,itzd,
C$OMP+ ucosa,usina,ucosc,usinc,ucosd,usind,lrota,lrotc,lrotd,
C$OMP+ xt,yt,zt,xu,yu,zu,xtu,ytu,ztu,rt2,ru2,rtru,rcb,cosine,sine,
C$OMP+ v1,c1,s1,v2,s2,c2,v3,c3,s3,cosine2,sine2,sine3,cosine3,phi1,
C$OMP+ phi2,phi3,dphi1,dphi2,dphi3,e,dedphi,dedxt,dedyt,dedzt,dedxu,
C$OMP+ dedyu,dedzu,dedxia,dedyia,ipr,
C$OMP+ dedzia,dedxib,dedyib,dedzib,dedxic,dedyic,dedzic,dedxid,
C$OMP+ dedyid,dedzid,vxx,vyx,vzx,vyy,vzy,vzz) DEFAULT(SHARED)

      ipr=OMP_GET_THREAD_NUM()+1
      do 101 i=nsl(ipr),nel(ipr)
      ia = iitors(1,i)
      ib = iitors(2,i)
      ic = iitors(3,i)
      id = iitors(4,i)
c
c     decide whether to compute the current interaction
      proceed = .true.
      if (use_group)  call groups (proceed,fgrp,ia,ib,ic,id,0,0)
      if (proceed)  proceed = (use(ia) .or. use(ib) .or.
     &                              use(ic) .or. use(id))
c
c     compute the value of the torsional angle
c
      if (proceed) then
       call ass12(xia,x(ia),yia,y(ia),zia,z(ia),
     1            xib,x(ib),yib,y(ib),zib,z(ib),
     2            xic,x(ic),yic,y(ic),zic,z(ic),
     3            xid,x(id),yid,y(id),zid,z(id))
       call ass12(xba,xib-xia,yba,yib-yia,zba,zib-zia,
     1            xcb,xic-xib,ycb,yic-yib,zcb,zic-zib,
     2            xdc,xid-xic,ydc,yid-yic,zdc,zid-zic,
     3            xab,xia-xib,yab,yia-yib,zab,zia-zib)
       call ass3( xdb,xid-xib,ydb,yid-yib,zdb,zid-zib)
       call ass3( xca,xic-xia,yca,yic-yia,zca,zic-zia)
       call ass3( xdb,xid-xib,ydb,yid-yib,zdb,zid-zib)

c      ## Petr Bour, bamd:
       if(nper.ne.0)then
c       relate a c d to b, find how much to transl and rotate:
c       transform a c d and get new distances
        call ass3(xtt,xia,ytt,yia,ztt,zia)
        call imaget(xab,yab,zab,ia,ucosa,usina,itxa,itya,itza,
     1  xtt,ytt,ztt,xib,yib,zib)
        call ass3(xtt,xic,ytt,yic,ztt,zic)
        call imaget(xcb,ycb,zcb,ic,ucosc,usinc,itxc,ityc,itzc,
     1  xtt,ytt,ztt,xib,yib,zib)
        call ass3(xtt,xid,ytt,yid,ztt,zid)
        call imaget(xdb,ydb,zdb,id,ucosd,usind,itxd,ityd,itzd,
     1  xtt,ytt,ztt,xib,yib,zib)
        lrota=(icx.eq.1.and.itxa.ne.0).or.
     1        (icx.eq.2.and.itya.ne.0).or.
     1        (icx.eq.3.and.itza.ne.0)
        lrotc=(icx.eq.1.and.itxc.ne.0).or.
     1        (icx.eq.2.and.ityc.ne.0).or.
     1        (icx.eq.3.and.itzc.ne.0)
        lrotd=(icx.eq.1.and.itxd.ne.0).or.
     1        (icx.eq.2.and.ityd.ne.0).or.
     1        (icx.eq.3.and.itzd.ne.0)
        if(lrota)
     1  call newpos(icx,itxa,itya,itza,xia,yia,zia,usina,ucosa)
        if(lrotc)
     1  call newpos(icx,itxc,ityc,itzc,xic,yic,zic,usinc,ucosc)
        if(lrotd)
     1  call newpos(icx,itxd,ityd,itzd,xid,yid,zid,usind,ucosd)
c       get updated distances,rest
        call ass3(xba,xib-xia,yba,yib-yia,zba,zib-zia)
        call ass3(xcb,xic-xib,ycb,yic-yib,zcb,zic-zib)
        call ass3(xdc,xid-xic,ydc,yid-yic,zdc,zid-zic)
        call ass3(xca,xic-xia,yca,yic-yia,zca,zic-zia)
        call ass3(xdb,xid-xib,ydb,yid-yib,zdb,zid-zib)
       else
        if (use_polymer) then
         call image (xba,yba,zba)
         call image (xcb,ycb,zcb)
         call image (xdc,ydc,zdc)
         call image (xca,yca,zca)
         call image (xdb,ydb,zdb)
       endif
      end if
      xt = yba*zcb - ycb*zba
      yt = zba*xcb - zcb*xba
      zt = xba*ycb - xcb*yba
      xu = ycb*zdc - ydc*zcb
      yu = zcb*xdc - zdc*xcb
      zu = xcb*ydc - xdc*ycb
      xtu = yt*zu - yu*zt
      ytu = zt*xu - zu*xt
      ztu = xt*yu - xu*yt
      rt2 = xt*xt + yt*yt + zt*zt
      ru2 = xu*xu + yu*yu + zu*zu
      rtru = sqrt(rt2 * ru2)
      if (rtru .ne. 0.0d0) then
       rcb = sqrt(xcb*xcb + ycb*ycb + zcb*zcb)
       cosine = (xt*xu + yt*yu + zt*zu) / rtru
       sine = (xcb*xtu + ycb*ytu + zcb*ztu) / (rcb*rtru)
c
c      set the improper torsional parameters for this angle
       v1 = itors1(1,i)
       c1 = itors1(3,i)
       s1 = itors1(4,i)
       v2 = itors2(1,i)
       c2 = itors2(3,i)
       s2 = itors2(4,i)
       v3 = itors3(1,i)
       c3 = itors3(3,i)
       s3 = itors3(4,i)
c
c      compute the multiple angle trigonometry and the phase terms
       cosine2 = cosine*cosine - sine*sine
       sine2 = 2.0d0 * cosine * sine
       cosine3 = cosine*cosine2 - sine*sine2
       sine3 = cosine*sine2 + sine*cosine2
       phi1 = 1.0d0 + (cosine*c1 + sine*s1)
       phi2 = 1.0d0 + (cosine2*c2 + sine2*s2)
       phi3 = 1.0d0 + (cosine3*c3 + sine3*s3)
       dphi1 = (cosine*s1 - sine*c1)
       dphi2 = 2.0d0 * (cosine2*s2 - sine2*c2)
       dphi3 = 3.0d0 * (cosine3*s3 - sine3*c3)
c
c      calculate improper torsion energy and master chain rule term
       e = itorunit * (v1*phi1+v2*phi2+v3*phi3)
       dedphi = itorunit * (v1*dphi1+v2*dphi2+v3*dphi3)
c
c      scale the interaction based on its group membership
       if (use_group) then
        e = e * fgrp
        dedphi = dedphi * fgrp
       end if
c
c      chain rule terms for first derivative components
       dedxt = dedphi * (yt*zcb - ycb*zt) / (rt2*rcb)
       dedyt = dedphi * (zt*xcb - zcb*xt) / (rt2*rcb)
       dedzt = dedphi * (xt*ycb - xcb*yt) / (rt2*rcb)
       dedxu = -dedphi * (yu*zcb - ycb*zu) / (ru2*rcb)
       dedyu = -dedphi * (zu*xcb - zcb*xu) / (ru2*rcb)
       dedzu = -dedphi * (xu*ycb - xcb*yu) / (ru2*rcb)
c
c      compute first derivative components for this angle
       dedxia = zcb*dedyt - ycb*dedzt
       dedyia = xcb*dedzt - zcb*dedxt
       dedzia = ycb*dedxt - xcb*dedyt
       dedxib = yca*dedzt - zca*dedyt + zdc*dedyu - ydc*dedzu
       dedyib = zca*dedxt - xca*dedzt + xdc*dedzu - zdc*dedxu
       dedzib = xca*dedyt - yca*dedxt + ydc*dedxu - xdc*dedyu
       dedxic = zba*dedyt - yba*dedzt + ydb*dedzu - zdb*dedyu
       dedyic = xba*dedzt - zba*dedxt + zdb*dedxu - xdb*dedzu
       dedzic = yba*dedxt - xba*dedyt + xdb*dedyu - ydb*dedxu
       dedxid = zcb*dedyu - ycb*dedzu
       dedyid = xcb*dedzu - zcb*dedxu
       dedzid = ycb*dedxu - xcb*dedyu
c
c      ia,ic,id  rotate back
       if(nper.ne.0)then
         if(icx.eq.1)then
          if(itxa.ne.0)call trb(dedyia,dedzia,ucosa,-usina)
          if(itxc.ne.0)call trb(dedyic,dedzic,ucosc,-usinc)
          if(itxd.ne.0)call trb(dedyid,dedzid,ucosd,-usind)
         endif
         if(icx.eq.2)then
          if(itya.ne.0)call trb(dedzia,dedxia,ucosa,-usina)
          if(ityc.ne.0)call trb(dedzic,dedxic,ucosc,-usinc)
          if(ityd.ne.0)call trb(dedzid,dedxid,ucosd,-usind)
         endif
         if(icx.eq.3)then
          if(itza.ne.0)call trb(dedxia,dedyia,ucosa,-usina)
          if(itzc.ne.0)call trb(dedxic,dedyic,ucosc,-usinc)
          if(itzd.ne.0)call trb(dedxid,dedyid,ucosd,-usind)
         endif               
         call incrtwistp(m_p(ipr),m_m(ipr),
     1   dedxia,dedyia,dedzia,x(ia),y(ia),z(ia),icx)
         call incrtwistp(m_p(ipr),m_m(ipr),
     1   dedxib,dedyib,dedzib,x(ib),y(ib),z(ib),icx)
         call incrtwistp(m_p(ipr),m_m(ipr),
     1   dedxic,dedyic,dedzic,x(ic),y(ic),z(ic),icx)
         call incrtwistp(m_p(ipr),m_m(ipr),
     1   dedxid,dedyid,dedzid,x(id),y(id),z(id),icx)
        endif               
c
        e_p(ipr) = e_p(ipr) + e
        i3=3*(ia-1)
        g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxia
        g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyia
        g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzia
        i3=3*(ib-1)
        g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxib
        g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyib
        g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzib
        i3=3*(ic-1)
        g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxic
        g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyic
        g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzic
        i3=3*(id-1)
        g_p(ipr,i3+1)=g_p(ipr,i3+1)+ dedxid
        g_p(ipr,i3+2)=g_p(ipr,i3+2)+ dedyid
        g_p(ipr,i3+3)=g_p(ipr,i3+3)+ dedzid

c       increment the internal virial tensor components
        vxx = xcb*(dedxic+dedxid) - xba*dedxia + xdc*dedxid
        vyx = ycb*(dedxic+dedxid) - yba*dedxia + ydc*dedxid
        vzx = zcb*(dedxic+dedxid) - zba*dedxia + zdc*dedxid
        vyy = ycb*(dedyic+dedyid) - yba*dedyia + ydc*dedyid
        vzy = zcb*(dedyic+dedyid) - zba*dedyia + zdc*dedyid
        vzz = zcb*(dedzic+dedzid) - zba*dedzia + zdc*dedzid
        call incrvirp(maxproc,ipr,vir_p,vxx,vyx,vzx,vyy,vzy,vzz)
       end if
      end if
101   continue

C$OMP END PARALLEL


C$OMP PARALLEL DO PRIVATE(i,ipr,i3) DEFAULT(SHARED)
      do 202 i=1,n
      i3=3*(i-1)
      do 202 ipr=1,nproc
      deit(1,i)=deit(1,i)+g_p(ipr,i3+1)
      deit(2,i)=deit(2,i)+g_p(ipr,i3+2)
202   deit(3,i)=deit(3,i)+g_p(ipr,i3+3)

      do 200 ipr=1,nproc
      vir(1,1)=vir(1,1)+vir_p(ipr,1,1)
      vir(1,2)=vir(1,2)+vir_p(ipr,1,2)
      vir(1,3)=vir(1,3)+vir_p(ipr,1,3)
      vir(2,1)=vir(2,1)+vir_p(ipr,2,1)
      vir(2,2)=vir(2,2)+vir_p(ipr,2,2)
      vir(2,3)=vir(2,3)+vir_p(ipr,2,3)
      vir(3,1)=vir(3,1)+vir_p(ipr,3,1)
      vir(3,2)=vir(3,2)+vir_p(ipr,3,2)
      vir(3,3)=vir(3,3)+vir_p(ipr,3,3)
      twm=twm+m_m(ipr)
      twp=twp+m_p(ipr)
200   eit=eit+e_p(ipr)

      return
      end

