c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##################################################################
c     ##                                                              ##
c     ##  subroutine gradient  --  find energy & gradient components  ##
c     ##                                                              ##
c     ##################################################################
c
c
c     "gradient" calls subroutines to calculate the potential energy
c     and first derivatives with respect to Cartesian coordinates
c
c
      subroutine gradient (energy,derivs)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bound.i'
      include 'cutoff.i'
      include 'deriv.i'
      include 'energi.i'
      include 'inter.i'
      include 'iounit.i'
      include 'potent.i'
      include 'rigid.i'
      include 'vdwpot.i'
      include 'virial.i'
      include 'cross.i'
      include 'cfast.i'
      include 'cell.i'
      include 'boxes.i'
      integer i,j,i3
      real*8 energy,cutoff,one,gnrm,time0,time1
      real*8 derivs(3,n)
c
c
c     zero out each of the potential energy components
c
      eb = 0.0d0
      ea = 0.0d0
      eba = 0.0d0
      eub = 0.0d0
      eaa = 0.0d0
      eopb = 0.0d0
      eopd = 0.0d0
      eid = 0.0d0
      eit = 0.0d0
      et = 0.0d0
      ept = 0.0d0
      ebt = 0.0d0
      ett = 0.0d0
      ev = 0.0d0
      ec = 0.0d0
      ecd = 0.0d0
      ed = 0.0d0
      em = 0.0d0
      ep = 0.0d0
      er = 0.0d0
      es = 0.0d0
      elf = 0.0d0
      eg = 0.0d0
      ex = 0.0d0
c
c     zero out each of the first derivative components
c
C$OMP PARALLEL DO PRIVATE(i,j) DEFAULT(SHARED)
      do 1 i = 1, n
      do 1 j = 1, 3
      deb(j,i) = 0.0d0
      dea(j,i) = 0.0d0
      deba(j,i) = 0.0d0
      deub(j,i) = 0.0d0
      deaa(j,i) = 0.0d0
      deopb(j,i) = 0.0d0
      deopd(j,i) = 0.0d0
      deid(j,i) = 0.0d0
      deit(j,i) = 0.0d0
      det(j,i) = 0.0d0
      dept(j,i) = 0.0d0
      debt(j,i) = 0.0d0
      dett(j,i) = 0.0d0
      dev(j,i) = 0.0d0
      dec(j,i) = 0.0d0
      decd(j,i) = 0.0d0
      ded(j,i) = 0.0d0
      dem(j,i) = 0.0d0
      dep(j,i) = 0.0d0
      der(j,i) = 0.0d0
      des(j,i) = 0.0d0
      delf(j,i) = 0.0d0
      deg(j,i) = 0.0d0
1     dex(j,i) = 0.0d0
c
c     zero out the virial and the intermolecular energy
c
      do 2 i = 1, 3
      do 2 j = 1, 3
2     vir(j,i) = 0.0d0
      einter = 0.0d0
c
c     maintain any periodic boundary conditions
c
      if (use_bounds .and. .not.use_rigid)then
        call bounds
      endif
c
c     update the pairwise interaction neighbor lists
c
      if (use_list)  call nblist
c
c     remove any previous use of the replicates method
c
      cutoff = 0.0d0
      call replica (cutoff)
      call setfastcuts
c
c     many implicit solvation models require Born radii
c
      if (use_born)  call born
c
c     alter bond and torsion constants for pisystem
      if (use_orbit)  call picalc
c
c     find molecular centers
      call calmolcen

c     precalculate twist for each atom
      call atomtwist
c
c     call the local geometry energy and gradient routines
c     call CPU_TIME(time0)
      if (use_bond)  call ebond1
c     call CPU_TIME(time1)
c     time0=time1
      if (use_angle)  call eangle1
c     call CPU_TIME(time1)
c     write(6,*)'tangle ',time1-time0
c     time0=time1
      if (use_strbnd)  call estrbnd1
      if (use_urey)  call eurey1
      if (use_angang)  call eangang1
      if (use_opbend)  call eopbend1
      if (use_opdist)  call eopdist1
      if (use_improp)  call eimprop1
c     call CPU_TIME(time1)
c     write(6,*)'timtor ',time1-time0
c     time0=time1
      if (use_imptor)  call eimptor1
      if (use_tors)  call etors1
c     call CPU_TIME(time1)
c     write(6,*)'ttor ',time1-time0
c     time0=time1
      if (use_pitors)  call epitors1
      if (use_strtor)  call estrtor1
      if (use_tortor)  call etortor1
c
c     call the van der Waals energy and gradient routines
c
      call echargelj1(use_charge,use_vdw)
c     call CPU_TIME(time1)
c     write(6,*)'tlc ',time1-time0
c     time0=time1

      if (use_vdw) then
       if (vdwtyp .eq. 'BUCKINGHAM')  call ebuck1
       if (vdwtyp .eq. 'MM3-HBOND')  call emm3hb1
       if (vdwtyp .eq. 'BUFFERED-14-7')  call ehal1
       if (vdwtyp .eq. 'GAUSSIAN')  call egauss1
      end if
c
c     call the electrostatic energy and gradient routines
c
      if (use_chgdpl)  call echgdpl1
      if (use_dipole)  call edipole1
      if (use_mpole .or. use_polar)  call empole1
      if (use_rxnfld)  call erxnfld1
c
c     call any miscellaneous energy and gradient routines
c
      if (use_solv)  call esolv1
      if (use_metal)  call emetal1
      if (use_geom)  call egeom1
      if (use_extra)  call extra1
      if (nper.gt.0)  call ewall1
 
c     sum up to get the total energy and first derivatives
c
      esum = eb + ea + eba + eub + eaa + eopb + eopd + eid + eit
     &          + et + ept + ebt + ett + ev + ec + ecd + ed + em
     &          + ep + er + es + elf + eg + ex
      energy = esum
C$OMP PARALLEL DO PRIVATE(i,j) DEFAULT(SHARED)
      do 3 i = 1, n
      do 3 j = 1, 3
      desum(j,i) = deb(j,i) + dea(j,i) + deba(j,i)
     &                      + deub(j,i) + deaa(j,i) + deopb(j,i)
     &                      + deopd(j,i) + deid(j,i) + deit(j,i)
     &                      + det(j,i) + dept(j,i) + debt(j,i)
     &                      + dett(j,i) + dev(j,i) + dec(j,i)
     &                      + decd(j,i) + ded(j,i) + dem(j,i)
     &                      + dep(j,i) + der(j,i) + des(j,i)
     &                      + delf(j,i) + deg(j,i) + dex(j,i)
3     derivs(j,i) = desum(j,i)

      if(lwgr)then
      write(6,*)'g'
       call writegr('GRAD.OUT',derivs,n)
      write(6,*)'G'
       write(6,*)'eb:',eb
       write(6,*)'ea:',ea
       write(6,*)'et:',et
       write(6,*)'eit:',eit
       write(6,*)'ev:',ev
       write(6,*)'ec:',ec
       write(6,*)'ex:',ex
       write(6,*)'gradient norms:'
       write(6,*)'b ',gnrm(deb ,n,maxatm)
       write(6,*)'a ',gnrm(dea ,n,maxatm)
       write(6,*)'t ',gnrm(det ,n,maxatm)
       write(6,*)'i ',gnrm(deit ,n,maxatm)
       write(6,*)'v ',gnrm(dev ,n,maxatm)
       write(6,*)'c ',gnrm(dec ,n,maxatm)
       write(6,*)'x ',gnrm(dex ,n,maxatm)
      endif

      goto 99
      i3=661
      write(6,*)' ato of interest:',i3
      write(6,*)'dea:',ea
      write(6,*)(dea(i,i3),i=1,3)
      write(6,*)'deb:',eb
      write(6,*)(deb(i,i3),i=1,3)
      write(6,*)'det:',et
      write(6,*)(det(i,i3),i=1,3)
      write(6,*)'deit:',eit
      write(6,*)(deit(i,i3),i=1,3)
      write(6,*)'dev:',ev
      write(6,*)(dev(i,i3),i=1,3)
      write(6,*)'dec:',ec
      write(6,*)(dec(i,i3),i=1,3)

      goto 99
      write(6,*) eb , ea , eba , eub , eaa , eopb , eopd , eid , eit
     &          , et , ept , ebt , ett , ev , ec , ecd , ed , em
     &          , ep , er , es , elf , eg , ex
c     stop
      one=1.0d0
      if(gnrm(deb ,n,maxatm).gt.one)write(6,*)'deb:'
      if(gnrm(deb ,n,maxatm).gt.one)call writegr('GRAD.OUT',deb,n)
      if(gnrm(dea ,n,maxatm).gt.one)write(6,*)'dea:'
      if(gnrm(dea ,n,maxatm).gt.one)call writegr('GRAD.OUT',dea,n)
      if(gnrm(deba,n,maxatm).gt.one)write(6,*)'deba:'
      if(gnrm(deba,n,maxatm).gt.one)call writegr('GRAD.OUT',deba,n)
      if(gnrm(deub,n,maxatm).gt.one)write(6,*)'deub:'
      if(gnrm(deub,n,maxatm).gt.one)call writegr('GRAD.OUT',deub,n)
      if(gnrm(deaa,n,maxatm).gt.one)write(6,*)'deaa:'
      if(gnrm(deaa,n,maxatm).gt.one)call writegr('GRAD.OUT',deaa,n)
      if(gnrm(deopb,n,maxatm).gt.one)write(6,*)'deopb:'
      if(gnrm(deopb,n,maxatm).gt.one)call writegr('GRAD.OUT',deopb,n)
      if(gnrm(deid,n,maxatm).gt.one)write(6,*)'deid:'
      if(gnrm(deid,n,maxatm).gt.one)call writegr('GRAD.OUT',deid,n)
      if(gnrm(deit,n,maxatm).gt.one)write(6,*)'deit:'
      if(gnrm(deit,n,maxatm).gt.one)call writegr('GRAD.OUT',deit,n)
      if(gnrm(det ,n,maxatm).gt.one)write(6,*)'det:'
      if(gnrm(det ,n,maxatm).gt.one)call writegr('GRAD.OUT',det,n)
      if(gnrm(dept,n,maxatm).gt.one)write(6,*)'dept:'
      if(gnrm(dept,n,maxatm).gt.one)call writegr('GRAD.OUT',dept,n)
      if(gnrm(debt,n,maxatm).gt.one)write(6,*)'debt:'
      if(gnrm(debt,n,maxatm).gt.one)call writegr('GRAD.OUT',debt,n)
      if(gnrm(dett,n,maxatm).gt.one)write(6,*)'dett:'
      if(gnrm(dett,n,maxatm).gt.one)call writegr('GRAD.OUT',dett,n)
      if(gnrm(dev ,n,maxatm).gt.one)write(6,*)'dev:'
      if(gnrm(dev ,n,maxatm).gt.one)call writegr('GRAD.OUT',dev,n)
      if(gnrm(dec ,n,maxatm).gt.one)write(6,*)'dec:'
      if(gnrm(dec ,n,maxatm).gt.one)call writegr('GRAD.OUT',dec,n)
      if(gnrm(decd,n,maxatm).gt.one)write(6,*)'decd:'
      if(gnrm(decd,n,maxatm).gt.one)call writegr('GRAD.OUT',decd,n)
      if(gnrm(ded ,n,maxatm).gt.one)write(6,*)'ded:'
      if(gnrm(ded ,n,maxatm).gt.one)call writegr('GRAD.OUT',ded,n)
      if(gnrm(dem ,n,maxatm).gt.one)write(6,*)'dem:'
      if(gnrm(dem ,n,maxatm).gt.one)call writegr('GRAD.OUT',dem,n)
      if(gnrm(dep ,n,maxatm).gt.one)write(6,*)'dep:'
      if(gnrm(dep ,n,maxatm).gt.one)call writegr('GRAD.OUT',dep,n)
      if(gnrm(der ,n,maxatm).gt.one)write(6,*)'der:'
      if(gnrm(der ,n,maxatm).gt.one)call writegr('GRAD.OUT',der,n)
      if(gnrm(des ,n,maxatm).gt.one)write(6,*)'des:'
      if(gnrm(des ,n,maxatm).gt.one)call writegr('GRAD.OUT',des,n)
      if(gnrm(delf,n,maxatm).gt.one)write(6,*)'delf:'
      if(gnrm(delf,n,maxatm).gt.one)call writegr('GRAD.OUT',delf,n)
      if(gnrm(deg ,n,maxatm).gt.one)write(6,*)'deg:'
      if(gnrm(deg ,n,maxatm).gt.one)call writegr('GRAD.OUT',deg,n)
      if(gnrm(dex ,n,maxatm).gt.one)write(6,*)'dex:'
      if(gnrm(dex ,n,maxatm).gt.one)call writegr('GRAD.OUT',dex,n)
99    continue
c
c     check for an illegal value for the total energy
c
      if (isnan(esum)) then
         write (iout,10)
   10    format (/,' GRADIENT  --  Illegal Value for the Total',
     &              ' Potential Energy')
         call fatal
      end if
      return
      end

      function gnrm(deb,n,maxatm)
      implicit none
      integer n,maxatm,i,j
      real*8 gnrm,deb(3,maxatm),f
      f=0.0d0
      do 1 i=1,n
      do 1 j=1,3
1     f=f+deb(j,i)**2
      gnrm=dsqrt(f)
      return
      end

      subroutine atomtwist
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'cross.i'
      include 'ctwist.i'
      real*8 argi,fia,farg,fargi
      integer i,ix
c     precalculate twist angle and damping factor for each atom:
      if(nper.ne.0)then
C$OMP  PARALLEL DO PRIVATE(i,argi,fia) DEFAULT(SHARED)
       do 13 i=1,n
       fia=0.0d0
       argi=farg(i,fia)
       ffarg(i)=fia
       aaarg(i)=argi
       do 13 ix=1,nx0
       argis(i+maxatm*(ix-1))=sin(argi*dble(ix))
13     argic(i+maxatm*(ix-1))=cos(argi*dble(ix))
      endif
      return
      end
