c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine mechanic  --  initialize molecular mechanics  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "mechanic" sets up needed parameters for the potential energy
c     calculation and reads in many of the user selectable options
c
c
      subroutine mechanic
      implicit none
      include 'sizes.i'
      include 'cutoff.i'
      include 'inform.i'
      include 'iounit.i'
      include 'potent.i'
      include 'vdwpot.i'
      include 'cfast.i'
      include 'atoms.i'
      include 'parallel.i'
      integer i

c
c
c     set the bonded connectivity lists and active atoms
c
      call attach
c
c     attach within the box only:
      call attach1

      call active
c
c     find bonds, angles, torsions, bitorsions and small rings
c
      call bonds
      call angles
      call torsions
      call bitors
      call rings
c
c     get the base force field from parameter file and keyfile
c
      call field
c
c     find unit cell type, lattice parameters and cutoff values
c
c     get periodic boundary conditions from keywords:
      call unitcell
c     set up periodic boundary conditions:
      call lattice
      call polymer
      call cutoffs
c
c     setup needed for potential energy smoothing methods
c
      call flatten
c
c     assign atom types, classes and other atomic information
c
      call katom
c
c     assign atoms to molecules and set the atom groups
c
      call molecule
      call atominmol
      call cluster
c
c     find any pisystem atoms, bonds and torsional angles
c
      call orbital
c
c     assign bond, angle and cross term potential parameters
c
      if (use_bond .or. use_strbnd .or. use_strtor
     &    .or. (use_vdw .and. vdwtyp.eq.'MM3-HBOND'))  call kbond
      if (use_angle .or. use_strbnd .or. use_angang)  call kangle
      if (use_strbnd)  call kstrbnd
      if (use_urey)  call kurey
      if (use_angang)  call kangang
c
c     assign out-of-plane deformation potential parameters
c
      if (use_angle .or. use_opbend)  call kopbend
      if (use_angle .or. use_opdist)  call kopdist
      if (use_improp)  call kimprop
      if (use_imptor)  call kimptor
c
c     assign torsion and torsion cross term potential parameters
c
      if (use_tors .or. use_strtor .or. use_tortor)  call ktors
      if (use_pitors)  call kpitors
      if (use_strtor)  call kstrtor
      if (use_tortor)  call ktortor
c
c     assign van der Waals and electrostatic potential parameters
c
      if (use_vdw .or. use_solv)  call kvdw
      if (use_charge .or. use_chgdpl .or. use_solv)  call kcharge
      if (use_dipole .or. use_chgdpl)  call kdipole
      if (use_mpole .or. use_polar .or.
     &    use_solv .or. use_rxnfld)  call kmpole
      if (use_polar .or. use_solv)  call kpolar
      if (use_ewald)  call kewald
c
c     assign solvation, metal, pisystem and restraint parameters
c
      if (use_solv)  call ksolv
      if (use_metal)  call kmetal
      if (use_orbit)  call korbit
      if (use_geom)  call kgeom
      if (use_extra)  call kextra
c
c     set hybrid parameter values for free energy perturbation
c
      call mutate
c
c     quit if essential parameter information is missing
c
      if (abort) then
         write (iout,10)
   10    format (/,' MECHANIC  --  Some Required Potential Energy',
     &              ' Parameters are Undefined')
         call fatal
      end if


c     set up parallel parameters:
      if(nproc.gt.maxproc)then
       write(iout,*)nproc,maxproc
       write(iout,*) 'recompile with larger maxproc'
       call fatal
      endif
      if(nproc.gt.omp_get_num_procs())then
       write(iout,*)nproc,omp_get_num_procs()
       write(iout,*) 'not enough processors'
       call fatal
      endif
      if(nproc.gt.omp_get_max_threads())
     1write(iout,*)'Number of threads will increase'
      call omp_set_num_threads(nproc)
      call omp_set_dynamic(0)
      call omp_set_dynamic(0)
      call omp_set_nested(0)
c     precalculate parallel distributions for "non-parralezable" loops:
      call ppd(nproc,maxproc,nstart,nend,n)
      write(6,5656)
5656  format(/,' Parallel distribution, linear:')
      do 1 i=1,nproc
1     write(iout,*)i,nstart(i),nend(i)
      write(6,*)'(forgotten)'
      call ppd2(nproc,maxproc,nstart,nend,n)

c     inital twist parameters (then they are updated in gradient):
      call atomtwist

      return
      end

      subroutine ppd(nproc,nproc0,nstart,nend,N)
      implicit none
      integer*4 nproc,nproc0,nstart(*),nend(*),nd,N,nrest,ipr
c
      nd=N/nproc
      nrest=N-nd*nproc
      nstart(1)=1
      nend(1)=nd
      if(nrest.ge.1)nend(1)=nend(1)+1
      do 7 ipr=2,nproc
      nstart(ipr)=nend(ipr-1)+1
      nend(ipr)=nstart(ipr)+nd-1
7     if(nrest.ge.ipr)nend(ipr)=nend(ipr)+1
      return
      end

      subroutine ppd2(nproc,nproc0,nstart,nend,N)
c     paralle distribution for i=1,n,k=i,n
      implicit none
      integer*4 nproc,nproc0,nstart(*),nend(*),nd,N,nrest,ipr,Ntot,i1,
     1i2,ntcon,ndif
      real*8 A,B,C
c
c     total number of elements:
      Ntot=(N*(N+1))/2
      nd=Ntot/nproc
      write(6,6001)Ntot,nproc,nd
6001  format(/,
     1       ' Parallel distribution, quadratic:',/,
     1' Number of elements  :',i20,/,
     1       ' Number of processors:',i20,/,
     1       ' Average chunk       :',i20,/)
      i1=1
      ntcon=0
      do 1 ipr=1,nproc
      A=1.0d0
      B=dble(-2*N-1)
      C=dble(-2*N+(2*N+3)*i1-i1**2-2+2*nd)
      i2=int((-B-dsqrt(B**2-4.0d0*A*C))/2.0d0)
      nstart(ipr)=i1
      if(ipr.eq.nproc)i2=N
      nend(ipr)=i2
      ndif=((N-i1+1)*(N-i1+2)-(N-i2)*(N-i2+1))/2
      write(6,6101)ipr,i1,i2,ndif
      ntcon=ntcon+ndif
6101  format(' Thread start end ndif: ',i3,2i10,i20)
1     i1=i2+1
      write(6,*)'Checksum:',ntcon
      write(6,*)
      return
      end
